
# 20092010.Meetstrategien [http://studiegids.uva.nl/web/uva/sgs/nl/c/7185.html]
# Interpolation of daily precipitation in the Netherlands

library(rgdal)
library(maptools)
library(gstat)
library(MASS)

# locaton of stations:
NL_stations_XY <- read.csv("NL_stations_XY.csv")
str(NL_stations_XY)
# fix the coordinates:
NL_stations_XY$X <- NL_stations_XY$X.X..km. * 1000
NL_stations_XY$Y <- NL_stations_XY$X.Y..km. * 1000
# fix the station names:
NL_stations_XY$Locatie <- as.factor(gsub(pattern='[[:space:]]+$', replacement='', NL_stations_XY$Locatie))

# import the precipitation values:
tmp <- readLines(paste(getwd(), "/", "monv_200906.txt", sep=""))
tmp <- gsub(pattern=' . ', replacement=' 0.0 ', tmp, fixed = TRUE)
tmp <- gsub(pattern='[[:space:]]{2,7}', replacement=',', tmp)
write.table(t(matrix(unlist(strsplit(tmp, ",")[-c(1:8)]), nrow=41)), "tmp.txt", col.names=FALSE, row.names=FALSE)
monv_200906 <- read.table("tmp.txt", col.names=c("empty1", "NR", paste("D", 1:30, sep=""), "I", "NORM", "II", "NORM", "III", "NORM", "MND", "NORM", "Locatie"))
monv_200906$empty1 <- NULL
monv_200906$Locatie <- as.factor(gsub(pattern='[[:space:]]+$', replacement='', monv_200906$Locatie))
str(monv_200906)

# merge the precipitation and coordinates:
monv_200906.XY <- merge(x=NL_stations_XY[,c("X", "Y", "Locatie")], y=monv_200906, by=c("Locatie"), all.x=TRUE)
str(monv_200906.XY)
coordinates(monv_200906.XY) <- ~X+Y

# import gridded map:
grids1km <- readGDAL("prec6.asc")
names(grids1km) <- "prec6"
grids1km$NLmask <- as.factor(readGDAL("NLmask.asc")$band1)

# CBS Wijk en buurtkaart 2008
download.file("http://www.cbs.nl/NR/rdonlyres/3D8F1FF7-E6BA-430D-AF3B-9F16CB6647B9/0/buurt_2008_gen1.zip", destfile=paste(getwd(), "/", "NL_buurtkaart2008.zip", sep=""))
unzip("NL_buurtkaart2008.zip")
NLprovs <- readShapePoly("gem_2008_gn2.shp", proj4string=CRS("+proj=sterea +lat_0=52.15616055555555 +lon_0=5.38763888888889 +k=0.999908 +x_0=155000 +y_0=463000 +ellps=bessel +towgs84=565.237,50.0087,465.658,-0.406857,0.350733,-1.87035,4.0812 +units=m +no_defs"))
NLprovs <- as(NLprovs, "SpatialLinesDataFrame")

# 1. How much rain did fall on 10th of June 2009 on the location: X=155000, Y=463000? 

sel <- !is.na(monv_200906.XY$D10)
bubble(monv_200906.XY[sel, "D10"])
# percipitation values are commonly skewed:
hist(log1p(monv_200906.XY$D10))
D10.svar <- variogram(log1p(D10)~1, monv_200906.XY[sel,])
D10.vgm <- fit.variogram(D10.svar, model=vgm(nugget=var(log1p(monv_200906.XY$D10), na.rm=TRUE)/3, model="Lin"))
# plot(D10.svar, D10.vgm)
New.XY <- data.frame(X=155000, Y=463000)
coordinates(New.XY) <- ~X+Y
expm1(krige(log1p(D10)~1, monv_200906.XY[sel,], New.XY, D10.vgm)$var1.pred)
# 1.32 mm

# 2. What is the uncertainty of that estimate for the same location? What is the overall uncertainty of the estimated rainfall for the whole area of the Netherlands?

# 95% range of predictions:
New.OK <- krige(log1p(D10)~1, monv_200906.XY[sel,], New.XY, D10.vgm) 
expm1(New.OK$var1.pred + 1.96*sqrt(New.OK$var1.var)); expm1(New.OK$var1.pred - 1.96*sqrt(New.OK$var1.var))
# 0.24 -- 3.32 mm
D10.OK <- krige(log1p(D10)~1, monv_200906.XY[sel,], grids1km["NLmask"], D10.vgm, nmin=10, nmax=50)
D10.OK$pred <- expm1(D10.OK$var1.pred)
D10ok.plt <- spplot(D10.OK["pred"], col.regions=rev(bpy.colors(40)), at=expm1(seq(0, max(log1p(D10.OK$pred), na.rm=TRUE), by=(max(log1p(D10.OK$pred), na.rm=TRUE)-0)/40)), sp.layout=list("sp.points", monv_200906.XY, pch="+", col="black"), main="Neerslag 9-10 juni 2009")
D10var.plt <- spplot(D10.OK["var1.var"], col.regions=grey(seq(0,1,by=0.025)), sp.layout=list("sp.points", monv_200906.XY, pch="+", col="red"), main="OK variance")
print(D10ok.plt, split=c(1,1,2,1), more=TRUE)
print(D10var.plt, split=c(2,1,2,1), more=FALSE)
mean(expm1(D10.OK$var1.pred + 1.96*sqrt(D10.OK$var1.var))-expm1(D10.OK$var1.pred - 1.96*sqrt(D10.OK$var1.var)), na.rm=TRUE)/2
# +/- 2
sd(monv_200906.XY$D10, na.rm=TRUE)

# 3. Could we use the map of long-term precipitation average to improve spatial interpolation of precipitation on June 8th?

monv_200906.XY.ov <- overlay(grids1km, monv_200906.XY)
monv_200906.XY.ov@data <- cbind(monv_200906.XY.ov@data, monv_200906.XY@data)
scatter.smooth(monv_200906.XY.ov$prec6, monv_200906.XY.ov$D10, xlab="Long-term precipitation (mm)", ylab="Precipitation on 10th of June") 
summary(lm(log1p(D10)~prec6, monv_200906.XY.ov))
# YES!

# 4. Up to which distance are the precipitation values auto-correlated in average? (plot at least 4 variograms)

D5.vgm <- fit.variogram(variogram(log1p(D5)~1, monv_200906.XY[!is.na(monv_200906.XY$D5),]), model=vgm(nugget=0, model="Exp", sill=var(log1p(monv_200906.XY$D5), na.rm=TRUE), range=50000))
D15.vgm <- fit.variogram(variogram(log1p(D15)~1, monv_200906.XY[!is.na(monv_200906.XY$D15),]), model=vgm(nugget=0, model="Exp", sill=var(log1p(monv_200906.XY$D15), na.rm=TRUE), range=50000))
D20.vgm <- fit.variogram(variogram(log1p(D20)~1, monv_200906.XY[!is.na(monv_200906.XY$D20),]), model=vgm(nugget=0, model="Exp", sill=var(log1p(monv_200906.XY$D20), na.rm=TRUE), range=50000))
# plot of variograms:
plot(variogramLine(D5.vgm, maxdist=2e+05)$dist, variogramLine(D5.vgm, maxdist=2e+05)$gamma, xlim=c(0, 2e+5), ylim=c(0, var(c(log1p(monv_200906.XY$D5), log1p(monv_200906.XY$D10), log1p(monv_200906.XY$D15), log1p(monv_200906.XY$D20)), na.rm=TRUE)), xlab="Distance (m)", ylab="Semivariance", type="l", col="black", lwd=2) 
lines(variogramLine(D10.vgm, maxdist=2e+05)$dist, variogramLine(D10.vgm, maxdist=2e+05)$gamma, col="red", lwd=2)
lines(variogramLine(D15.vgm, maxdist=2e+05)$dist, variogramLine(D15.vgm, maxdist=2e+05)$gamma, col="blue", lwd=2)
lines(variogramLine(D20.vgm, maxdist=2e+05)$dist, variogramLine(D20.vgm, maxdist=2e+05)$gamma, col="green", lwd=2)


# 5. At which location (city or village name) would you predict the maximum rainfall on June 8th? Is this value higher than what has been observed at the meteorological stations? 

D8.rvgm <- fit.variogram(variogram(log1p(D8)~prec6, monv_200906.XY.ov[!is.na(monv_200906.XY.ov$D8)&!is.na(monv_200906.XY.ov$prec6),]), model=vgm(nugget=0, model="Exp", sill=var(log1p(monv_200906.XY.ov$D8), na.rm=TRUE)/2, range=50000))
D8.RK <- krige(log1p(D8)~prec6, monv_200906.XY.ov[!is.na(monv_200906.XY.ov$D8)&!is.na(monv_200906.XY.ov$prec6),], grids1km, D8.rvgm, nmin=20, nmax=50)
# mask areas of interest:
D8.RK$pred <- ifelse(is.na(grids1km$NLmask), NA, expm1(D8.RK$var1.pred))
spplot(D8.RK["pred"], col.regions=rev(bpy.colors(40)), at=expm1(seq(0, max(log1p(D8.RK$pred), na.rm=TRUE), by=(max(log1p(D8.RK$pred), na.rm=TRUE)-0)/40)), sp.layout=list("sp.lines", NLprovs, col="black"), main="Neerslag 7-8 juni 2009")
write.asciigrid(D8.RK["pred"], "D8_RK.asc", na.value=-1)

D8.RK.pnt <- as(D8.RK["pred"], "SpatialPointsDataFrame")
proj4string(D8.RK.pnt) <- NLprovs@proj4string
spTransform(D8.RK.pnt[which.max(D8.RK.pnt$pred),], CRS("+proj=latlon +datum=WGS84"))
# 8939 (6.94032, 53.3179)  24.9 mm  # Farmsum
max(monv_200906.XY.ov$D8, na.rm=TRUE)

# end of script;